# configuration only for tests

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': 'ralph_test',
        'OPTIONS': dict(
        ),
    },
}

SECRET_KEY = "secret"



# -*- coding: utf-8 -*-
from collections import Container

from django.db.models import Model


def format_single_val_for_js(val):
    """Return the appropriate js representation of a single value."""
    # null, true and false are correct json values
    if not (val is None or isinstance(val, bool)):
        if isinstance(val, Model):
            val = val.pk
        if not isinstance(val, basestring):
            val = str(val)
    return val


def format_val_for_js(val):
    if not isinstance(val, basestring) and isinstance(val, Container):
        val = map(format_single_val_for_js, val)
    else:
        val = format_single_val_for_js(val)
    return val


def first_letter_lower(name):
    return name[0].lower() + name[1:]


class DependencyCondition(object):
    """Base abstract class for each dependency.

    It should have defined met function and get_js_format if it's unusual.
    """
    def met(self, value):
        """Return ``True`` if value met condition.

        :param value: master field value.
        """
        raise NotImplementedError

    def get_js_format(self):
        """Return list with condition definition."""
        return [first_letter_lower(self.__class__.__name__)]


class ArgumentedCondition(DependencyCondition):
    """Abstract class for condition with argument."""
    def __init__(self, value):
        self.value = value

    def get_js_format(self):
        format = super(ArgumentedCondition, self).get_js_format()
        format.append(format_val_for_js(self.value))
        return format


###############################################################################
# Real conditions here:
###############################################################################


class Any(DependencyCondition):
    """Condition for any ``master.value`` (even empty)."""
    def met(self, val):
        return True


class Exact(ArgumentedCondition):
    """Condition for ``master.value`` equals ``value``."""
    def met(self, val):
        return format_val_for_js(self.value) == format_val_for_js(val)


class MemberOf(ArgumentedCondition):
    """Condition for ``master.value`` is in ``value``."""
    def met(self, val):
        return format_val_for_js(val) in format_val_for_js(self.value)


class NotEmpty(DependencyCondition):
    """Condition for ``master.value`` is not empty."""
    def met(self, val):
        return val != "" and val is not None



# -*- coding: utf-8 -*-
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import logging

from django.conf import settings
from django.contrib import messages
from django.core.exceptions import ImproperlyConfigured
from django.db.models import Q
from django.forms.models import modelformset_factory
from django.http import HttpResponseRedirect
from django.utils.translation import ugettext_lazy as _
from django.views.generic import TemplateView


logger = logging.getLogger(__name__)


class BulkEditBase(TemplateView):
    """Base view for bulk edit. This class is very helpful - handle any formset
    from any model. Formset is automaticly generated from model or queryset.
    Additionally you must specified class form for forms which display in
    formset (in simplest case it is ``ModelForm``).

    Example of use:
    .. code-block:: python
        UserBulkEdit(BulkEditBase):
            template_name = 'bulk_edit.html'
            model = User
            form_bulk = UserForm

    or

    .. code-block:: python
        UserBulkEdit(BulkEditBase):
            template_name = 'bulk_edit.html'
            queryset = User.objects.filter(is_active=False)
            form_bulk = UserForm
    """
    commit_on_valid = True
    form_bulk = None
    http_method_names = ['get', 'post']
    ids_key_name = 'select'
    model = None
    queryset = None
    success_url = None
    invalid_url = None

    def initial_forms(self, formset, queryset):
        """Initial forms in formset before render in page. You can manipulate
        fields in forms (eg. change widget, set default value depends on
        queryset, etc).

        :param formset: formset with generated forms
        :param queryset:
        """

    def get(self, request, *args, **kwargs):
        max_bulk_edit_size = getattr(settings, 'MAX_BULK_EDIT_SIZE', 40)
        queryset = self.get_queryset(*args, **kwargs)
        objects_count = queryset.count()
        if not (0 < objects_count <= max_bulk_edit_size):
            if objects_count > max_bulk_edit_size:
                messages.warning(
                    request,
                    _('You can edit max {} items'.format(max_bulk_edit_size)),
                )
            elif not objects_count:
                messages.warning(request, _('Nothing to edit.'))
            return HttpResponseRedirect(self.get_invalid_url())
        FormSet = self.get_formset()
        formset = FormSet(queryset=queryset)
        self.initial_forms(formset, queryset)
        kwargs.update({'formset': formset})
        return self.render_to_response(self.get_context_data(*args, **kwargs))

    def post(self, request, *args, **kwargs):
        FormSet = self.get_formset()
        formset = FormSet(request.POST)
        if formset.is_valid():
            instances = formset.save(commit=self.commit_on_valid)
            if not self.commit_on_valid:
                self.save_formset(instances, formset)
            messages.success(request, _('Changes saved.'))
            return HttpResponseRedirect(self.get_success_url())
        self.handle_formset_error(formset.get_form_error())
        kwargs.update({'formset': formset})
        return self.render_to_response(self.get_context_data(*args, **kwargs))

    def get_context_data(self, *args, **kwargs):
        context = super(BulkEditBase, self).get_context_data(*args, **kwargs)
        context.update(kwargs)
        return context

    def get_success_url(self):
        """Redirect after successfully send formset.

        :return: returns URL (``str``)
        """
        if not self.success_url:
            return self.request.get_full_path()
        return self.success_url

    def get_invalid_url(self):
        """Redirect after send invalid or empty formset.

        :return: returns URL (``str``)
        """
        if not self.invalid_url:
            return '..'
        return self.invalid_url

    def get_form_bulk(self):
        """Attribute form_bulk is required. If yours form is lazy or dynamic
        generated you can override this method.

        :return: standard django form class
        """
        if not self.form_bulk:
            raise ImproperlyConfigured(
                'You must define %(cls)s.form_bulk or override '
                '%(cls)s.get_form_bulk method.' % {
                    'cls': self.__class__.__name__,
                }
            )
        return self.form_bulk

    def get_formset(self, **kwargs):
        kwargs.update({'form': self.get_form_bulk()})
        return modelformset_factory(self.model, extra=0, **kwargs)

    def get_queryset(self, *args, **kwargs):
        if not self.queryset:
            if self.model:
                query = self.get_query_from_request()
                return self.model._default_manager.filter(query)
            else:
                raise ImproperlyConfigured(
                    'You must define %(cls)s.model or %(cls)s.queryset or '
                    'override %(cls)s.get_queryset method.' % {
                        'cls': self.__class__.__name__,
                    }
                )
        return self.queryset._clone()

    def get_query_from_request(self):
        return Q(pk__in=self.get_items_ids())

    def get_items_ids(self):
        """Get ids of object from request parameters."""
        ids = self.request.GET.getlist(self.ids_key_name)
        try:
            int_ids = map(int, ids)
        except ValueError:
            int_ids = []
        return int_ids

    def save_formset(self, instances, formset):
        """If commit_on_valid is set to ``False`` then this method will be
        required. If you override this method you must save instances manually
        (if you want of course).

        .. note::
            Please use context manager ``transaction.commit_on_success()`` from
            ``from django.db import transaction`` where you save instances
            to database.

        :param instances: list of objects
        :param formset: formset
        """
        if not self.commit_on_valid:
            raise ImproperlyConfigured(
                'You must override %(cls)s.save_formset if attribute '
                '%(cls)s.commit_on_valid is set.' % {
                    'cls': self.__class__.__name__
                }
            )

    def handle_formset_error(self, formset_error):
        messages.error(self.request, formset_error)



# -*- coding: utf-8 -*-
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import logging
import json

from django.http import HttpResponse, HttpResponseBadRequest
from django.views.generic import View


logger = logging.getLogger(__name__)


class DependencyView(View):
    """
    Base view for dependencies ajax data providers.

    Request is checked for the presence of the master field value.
    Subclass must implement :meth:`get_values` which returns dict of
    dependent fields values, e.g.: Fields `sex` and `country`
    depend on `name`.

    Request:

        >>> {
            'value': 'Andrzej'
        }

    Response:

        >>> {
            'sex': 'man',
            'country': 'Poland'
        }
    """
    def get_values(self, value):
        """
        Should provide value/values of dependent fields.

        If value is not proper returns HttpResponse class which
        will be returned with proper code.

        :param value: master field value, comes with ajax request
        :return: map of dependent field names - fields values
        """
        raise NotImplementedError("Subclass should implement this method.")

    def post(self, *args, **kwargs):
        try:
            value = self.request.POST['value']
        except KeyError:
            logger.debug("dependencies\tEmpty field value.")
            return HttpResponseBadRequest("Value is not provided.")

        result = self.get_values(value)

        if isinstance(result, HttpResponse):
            return result

        try:
            assert isinstance(result, dict)
            response = json.dumps(result)
        except (TypeError, AssertionError):
            logger.error(
                "dependencies\tget_values should return parseable dict."
            )
            return HttpResponse(status=500)
        return HttpResponse(response, mimetype="application/json")



# -*- coding: utf-8 -*-

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from datetime import date
from mock import Mock

from django.test import TestCase
from django.db.models import Model

from bob.forms.dependency_conditions import (
    first_letter_lower,
    format_single_val_for_js,
    format_val_for_js,
    Any,
    Exact,
    MemberOf,
    NotEmpty,
)


class FirstLetterLowerTest(TestCase):
    def test_lower(self):
        self.assertEqual(first_letter_lower("abcdefgh"), "abcdefgh")
        self.assertEqual(first_letter_lower("a a a a"), "a a a a")

    def test_upper(self):
        self.assertEqual(first_letter_lower("AbDe3fd#EDFG"), "abDe3fd#EDFG")
        self.assertEqual(first_letter_lower("A a a a"), "a a a a")

    def test_real(self):
        self.assertEqual(first_letter_lower("Exact"), "exact")
        self.assertEqual(first_letter_lower("MemberOf"), "memberOf")
        self.assertEqual(first_letter_lower("NotEmpty"), "notEmpty")


class FormatSingleValForJsTest(TestCase):
    def test_bool(self):
        self.assertEqual(format_single_val_for_js(True), True)
        self.assertEqual(format_single_val_for_js(False), False)

    def test_none(self):
        self.assertEqual(format_single_val_for_js(None), None)

    def test_not_string(self):
        self.assertEqual(format_single_val_for_js("123"), "123")

    def test_date(self):
        dt = date(2012, 04, 15)
        self.assertEqual(format_single_val_for_js(dt), "2012-04-15")

    def test_model(self):
        model = Mock(spec=Model)
        model.pk = "321"
        self.assertEqual(format_single_val_for_js(model), "321")


class FormatValForJsTest(TestCase):
    def test_single_string(self):
        self.assertEqual(format_val_for_js("abcdefgh22"), "abcdefgh22")
        self.assertEqual(format_val_for_js("Pokoijiuh"), "Pokoijiuh")

    def test_single_bool(self):
        self.assertEqual(format_single_val_for_js(True), True)
        self.assertEqual(format_single_val_for_js(False), False)

    def test_simple_list(self):
        self.assertEqual(
            format_val_for_js([True, "456", None, 123]),
            [True, "456", None, "123"],
        )

    def test_list_with_date(self):
        dt = date(2012, 04, 15)
        self.assertEqual(
            format_val_for_js([True, "456", dt, 123]),
            [True, "456", "2012-04-15", "123"],
        )

    def test_list_with_model(self):
        model = Mock(spec=Model)
        model.pk = "321"
        self.assertEqual(
            format_val_for_js([1, "2", model]),
            ["1", "2", "321"],
        )


class AnyTest(TestCase):
    def test_python_condition(self):
        any_condition = Any()
        self.assertTrue(any_condition.met("sdf"))
        self.assertTrue(any_condition.met(False))
        self.assertTrue(any_condition.met(3))
        self.assertTrue(any_condition.met(date(2005, 10, 03)))
        self.assertTrue(any_condition.met(None))
        self.assertTrue(any_condition.met(""))

    def test_js_format_simple(self):
        any_condition = Any()
        self.assertEqual(any_condition.get_js_format(), ["any"])


class ExactTest(TestCase):
    def test_python_condition(self):
        exact = Exact(5)
        self.assertTrue(exact.met(5))
        self.assertTrue(exact.met("5"))
        self.assertFalse(exact.met(7))
        self.assertFalse(exact.met("3"))

    def test_python_condition_date(self):
        exact = Exact(date(2100, 10, 5))
        self.assertTrue(exact.met("2100-10-05"))
        self.assertTrue(exact.met(date(2100, 10, 5)))
        self.assertFalse(exact.met("2100-11-05"))
        self.assertFalse(exact.met(date(2000, 4, 18)))

    def test_python_condition_model(self):
        model_mock = Mock(spec=Model)
        model_mock.pk = 129
        model_mock_2 = Mock(spec=Model)
        model_mock_2.pk = 55
        exact = Exact(model_mock)
        self.assertTrue(exact.met(model_mock))
        self.assertTrue(exact.met("129"))
        self.assertTrue(exact.met(129))
        self.assertFalse(exact.met(model_mock_2))
        self.assertFalse(exact.met("185"))
        self.assertFalse(exact.met(19))

    def test_js_format_simple(self):
        exact = Exact(10)
        self.assertEqual(exact.get_js_format(), ["exact", "10"])

    def test_js_format_date(self):
        exact = Exact(date(2000, 01, 01))
        self.assertEqual(exact.get_js_format(), ["exact", "2000-01-01"])


class MemberOfTest(TestCase):
    def test_python_condition(self):
        memberOf = MemberOf([1, 2, 5, date(2010, 03, 19)])
        self.assertTrue(memberOf.met("2"))
        self.assertTrue(memberOf.met(5))
        self.assertFalse(memberOf.met(3))
        self.assertFalse(memberOf.met(None))

    def test_python_condition_date(self):
        memberOf = MemberOf([1, date(2010, 03, 19), 5])
        self.assertTrue(memberOf.met("2010-03-19"))
        self.assertTrue(memberOf.met(date(2010, 03, 19)))
        self.assertFalse(memberOf.met("2010-03-18"))
        self.assertFalse(memberOf.met(date(2010, 02, 19)))

    def test_python_condition_model(self):
        model_mock = Mock(spec=Model)
        model_mock.pk = 129
        exact = Exact(model_mock)
        self.assertTrue(exact.met(model_mock))
        self.assertTrue(exact.met("129"))
        self.assertTrue(exact.met(129))

    def test_js_format_simple(self):
        memberOf = MemberOf([1, True, "3"])
        self.assertEqual(
            memberOf.get_js_format(), ["memberOf", ["1", True, "3"]]
        )

    def test_js_format_date(self):
        memberOf = MemberOf([1, date(2010, 02, 19), "3"])
        self.assertEqual(
            memberOf.get_js_format(), ["memberOf", ["1", "2010-02-19", "3"]]
        )

    def test_js_format_model(self):
        model_mock = Mock(spec=Model)
        model_mock.pk = 129
        memberOf = MemberOf([True, model_mock, 0])
        self.assertEqual(
            memberOf.get_js_format(), ["memberOf", [True, "129", "0"]]
        )


class NotEmptyTest(TestCase):
    def test_python_condition(self):
        notEmpty = NotEmpty()
        self.assertTrue(notEmpty.met("srlktmgsrltkgm"))
        self.assertTrue(notEmpty.met(False))
        self.assertFalse(notEmpty.met(None))
        self.assertFalse(notEmpty.met(""))

    def test_js_format_simple(self):
        notEmpty = NotEmpty()
        self.assertEqual(
            notEmpty.get_js_format(), ["notEmpty"]
        )



# -*- coding: utf-8 -*-
"""All the objects needed to use the dependency mechanism.

For dependency usage you need to:

* add :class:`DependencyForm` as your form mixin,
* (optional) define :class:`bob.views.dependency.DependencyView` if
  :const:`AJAX_UPDATE` used,
* define ``dependencies`` property in you form,
* add tag to you form template:

    <form id="{{ form_id }}" method="POST" {% dependency_data asset_form %}>


``dependencies`` example:

.. code-block:: python

    @property
    def dependencies(self):
        return [
            Dependency(
                'slots',
                'category',
                dependency_conditions.MemberOf(
                    AssetCategory.objects.filter(is_blade=True).all()
                ),
                SHOW,
            ),
            Dependency(
                'location',
                'owner',
                dependency_conditions.NotEmpty(),
                AJAX_UPDATE,
                url=reverse('category_dependency_view'),
                page_load_update=False,
            )
        ]
"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from django.utils.translation import ugettext as _

from bob.forms.dependency_conditions import DependencyCondition


# dependencies types
AJAX_UPDATE = 'AJAX_UPDATE'
"""Load slave value by ajax request if master value met condition."""
CLONE = 'CLONE'
"""Copy master value to slave value."""
SHOW = 'SHOW'
"""Show slave field if master value met condition."""
REQUIRE = 'REQUIRE'
"""Set slave value required."""


class Dependency(object):
    """The single dependency.

    If action is :const:`AJAX_UPDATE` you have to set ``url`` kwarg. Url
    corresponds to :class:`bob.views.dependency.DependencyView` subclass.

    :param slave: Dependency target field.
    :param master: Dependency source field. It's value is condition param.
    :param condition: :class:`.dependency_conditions.DependencyCondition`
        subclass.
    :param action: One of :const:`AJAX_UPDATE`, :const:`CLONE`,
        :const:`SHOW`, :const:`REQUIRE`.
    :param \*\*options: Another options, like ``url`` if action is
        :const:`AJAX_UPDATE`.
    """

    def __init__(self, slave, master, condition, action, **options):
        self.slave = slave
        self.master = master
        self.condition = condition
        assert isinstance(condition, DependencyCondition)
        self.action = action
        self.options = options
        self.options['page_load_update'] = options.get(
            'page_load_update', True,
        )
        if action == AJAX_UPDATE:
            assert 'url' in options, "Source url not provided."

    def met(self, data):
        """Return ``True`` if condition is met.

        :param data: form.cleaned_data dictionary with master field value.
        """
        val = data.get(self.master)
        return self.condition.met(val)


class DependencyForm(object):  # Can't inherit Form due to metaclass conflict
    """Form mixin with dependency feature.

    It should be used in django forms which includes dependencies.
    It provides clean method with dependencies validation.
    """

    dependencies = []

    def clean(self):
        cleaned_data = super(DependencyForm, self).clean()
        for dep in self.dependencies:
            if dep.met(cleaned_data):
                if dep.action == REQUIRE:
                    unfilled = (
                        dep.slave in cleaned_data and not
                        cleaned_data[dep.slave]
                    )
                    if unfilled:
                        msg = _("This field is required")
                        self._errors[dep.slave] = self.error_class([msg])
            else:
                if dep.action in {SHOW}:
                    self._errors.pop(dep.slave, None)
        return cleaned_data

    def get_dependencies_for_js(self):
        """Return the dependencies in a format ready to be JSON serialized for
        JavaScript"""
        result = []
        for dep in self.dependencies:
            result.append({
                'slave': dep.slave,
                'master': dep.master,
                'condition': dep.condition.get_js_format(),
                'action': dep.action,
                'options': dep.options
            })
        return result
